@@ -8,6 +8,7 @@ import pyvo
 from time import sleep
 import argparse
 import sys
+import traceback
 
 from . import applicationPath, settingsFile
 from .version import __version__
@@ -24,12 +25,21 @@ from .theme import (
 from .examples import examplesList
 
 debugMode = False
+noEnumerationColumn = False
 
 mainWindowID = "main-window"
 queryTextID = "query-text"
 
 tabulateFloatfmtPrecision = "g"
 
+# Dear PyGui (and Dear ImGui) has a limitation of 64 columns in a table
+# https://dearpygui.readthedocs.io/en/latest/documentation/tables.html
+# https://github.com/ocornut/imgui/issues/2957#issuecomment-758136035
+# https://github.com/ocornut/imgui/pull/4876
+dpgColumnsMax = 64
+
+windowMinWidth = 900
+
 lastQueryResults = {}
 executingQuery = False
 
@@ -125,46 +135,83 @@ def executeQuery():
             print(f"[WARNING] Couldn't print results. {ex}")
 
     lastQueryResults = results.to_table().to_pandas()
-    # try:
-    with dpg.table(
-        parent="resultsGroup",
-        tag="resultsTable",
-        header_row=True,
-        resizable=True,
-        borders_outerH=True,
-        borders_innerV=True,
-        borders_innerH=True,
-        borders_outerV=True,
-        policy=dpg.mvTable_SizingStretchProp,
-        clipper=True
-    ):
-        dpg.add_table_column()
-        for header in lastQueryResults:
-            dpg.add_table_column(label=header)
-        for index, row in lastQueryResults.iterrows():
-            with dpg.table_row():
-                with dpg.table_cell():
-                    dpg.add_text(default_value=f"{index+1}")
-                cellIndex = 1
-                for cell in row:
-                    with dpg.table_cell():
-                        cellID = f"cell{index+1}{cellIndex}"
-                        dpg.add_text(
-                            tag=cellID,
-                            default_value=cell
-                        )
-                        dpg.bind_item_handler_registry(
-                            cellID,
-                            "cell-handler"
-                        )
-                    cellIndex += 1
-    # except Exception as ex:
-    #     errorMsg = "Couldn't generate the results table"
-    #     print(f"[ERROR] {errorMsg}. {ex}", file=sys.stderr)
-    #     dpg.set_value("errorMessage", errorMsg)
-    #     dpg.show_item("errorMessage")
-    #     showLoading(False)
-    #     return
+    rowsCount, columnsCount = lastQueryResults.shape
+    if debugMode:
+        print(f"[DEBUG] Columns: {columnsCount}, rows: {rowsCount}")
+    if columnsCount > dpgColumnsMax:
+        dpg.set_value(
+            "errorMessage",
+            " ".join((
+                "You have requested too many columns in your query.",
+                f"Dear PyGui only supports maximum {dpgColumnsMax} columns",
+                "in a table, and so trying to display results for your query",
+                "will crash the application. Remove some columns from your",
+                "SELECT statement and try again."
+            ))
+        )
+        dpg.show_item("errorMessage")
+        showLoading(False)
+        return
+    try:
+        # when there isn't that many columns,
+        # squeezed table doesn't look nice
+        addHorizontalScroll = (
+            dpg.get_item_width(mainWindowID) / columnsCount < 150
+        )
+        with dpg.table(
+            parent="resultsGroup",
+            tag="resultsTable",
+            header_row=True,
+            resizable=True,
+            borders_outerH=True,
+            borders_innerV=True,
+            borders_innerH=True,
+            borders_outerV=True,
+            clipper=True,
+            # row_background=True,
+            # freeze_rows=0,
+            # freeze_columns=1,
+            # scrollY=True,
+            policy=(
+                dpg.mvTable_SizingFixedSame
+                if addHorizontalScroll
+                else dpg.mvTable_SizingStretchProp
+            ),
+            scrollX=addHorizontalScroll
+        ):
+            if not noEnumerationColumn and rowsCount > 1:
+                dpg.add_table_column(label="#")
+            for header in lastQueryResults:
+                dpg.add_table_column(label=header)
+            for index, row in lastQueryResults.iterrows():
+                with dpg.table_row():
+                    if not noEnumerationColumn and rowsCount > 1:
+                        with dpg.table_cell():
+                            dpg.add_text(default_value=f"{index+1}")
+                    cellIndex = 1
+                    for cell in row:
+                        with dpg.table_cell():
+                            cellID = f"cell-{index+1}-{cellIndex}"
+                            dpg.add_text(
+                                tag=cellID,
+                                default_value=cell
+                            )
+                            dpg.bind_item_handler_registry(
+                                cellID,
+                                "cell-handler"
+                            )
+                        cellIndex += 1
+    except Exception as ex:
+        errorMsg = "Couldn't generate the results table"
+        print(f"[ERROR] {errorMsg}. {ex}", file=sys.stderr)
+        traceback.print_exc(file=sys.stderr)
+        dpg.set_value(
+            "errorMessage",
+            f"{errorMsg}. There might be more details in console/stderr."
+        )
+        dpg.show_item("errorMessage")
+        showLoading(False)
+        return
     showLoading(False)
     dpg.show_item("resultsGroup")
     dpg.configure_item("menuSaveFile", enabled=True)
@@ -206,7 +253,7 @@ def cellClicked(sender, app_data):
         cellValue = dpg.get_value(app_data[1])
         # print(cellValue)
         dpg.set_clipboard_text(cellValue)
-        dpg.set_value(app_data[1], "(copied)")
+        dpg.set_value(app_data[1], "[copied]")
         dpg.bind_item_theme(app_data[1], getCellHighlightedTheme())
         sleep(1)
         dpg.bind_item_theme(app_data[1], getCellDefaultTheme())
@@ -222,13 +269,14 @@ def showDPGabout():
 def main():
     global debugMode
     global tabulateFloatfmtPrecision
+    global noEnumerationColumn
 
     argParser = argparse.ArgumentParser(
         prog="tap-adql-sandbox",
         description=" ".join((
             "%(prog)s  Copyright (C) 2022  retif\nA",
             "sandbox application for executing ADQL queries",
-            f"via TAP interface."
+            "via TAP interface."
         )),
         formatter_class=argparse.RawDescriptionHelpFormatter,
         allow_abbrev=False
@@ -241,7 +289,15 @@ def main():
     argParser.add_argument(
         "--debug",
         action='store_true',
-        help=f"enable debug/dev mode (default: %(default)s)"
+        help="enable debug/dev mode (default: %(default)s)"
+    )
+    argParser.add_argument(
+        "--no-enum-column",
+        action='store_true',
+        help=" ".join((
+            "add artificial first column",
+            "to enumerate results (default: %(default)s)"
+        ))
     )
     argParser.add_argument(
         "--tbl-flt-prcs",
@@ -252,6 +308,7 @@ def main():
     # print(cliArgs)
 
     debugMode = cliArgs.debug
+    noEnumerationColumn = cliArgs.no_enum_column
     if cliArgs.tbl_flt_prcs:
         tabulateFloatfmtPrecision = cliArgs.tbl_flt_prcs
 
@@ -395,6 +452,8 @@ def main():
         dpg.add_text(
             tag="errorMessage",
             default_value="Error",
+            # https://github.com/hoffstadt/DearPyGui/issues/1275
+            wrap=windowMinWidth-50,
             show=False
         )
 
@@ -453,13 +512,13 @@ def main():
             ))
         )
         dpg.add_text(f"Version: {__version__}")
-        dpg.add_text(f"Source code: https://github.com/retifrav/tap-adql-sandbox")
-        dpg.add_text(f"License: GPLv3")
+        dpg.add_text("Source code: https://github.com/retifrav/tap-adql-sandbox")
+        dpg.add_text("License: GPLv3")
         dpg.add_spacer()
         dpg.add_separator()
         dpg.add_spacer(height=5)
         with dpg.group(horizontal=True):
-            dpg.add_text("Created with DearPyGui")
+            dpg.add_text("Created with Dear PyGui")
             dpg.add_button(
                 label="about that...",
                 callback=showDPGabout
@@ -505,7 +564,7 @@ def main():
         title="TAP ADQL sandbox",
         width=1200,
         height=800,
-        min_width=900,
+        min_width=windowMinWidth,
         min_height=600,
         small_icon=str(applicationPath/"icons/planet-128.ico"),
         large_icon=str(applicationPath/"icons/planet-256.ico")
